import 'core-js/modules/es.array.iterator.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import { jsx, jsxs } from 'react/jsx-runtime';
import { useState, useRef, useEffect } from 'react';
import 'core-js/modules/es.array.reduce.js';
import 'core-js/modules/es.json.stringify.js';
import 'core-js/modules/es.promise.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.regexp.test.js';
import 'core-js/modules/es.regexp.to-string.js';
import 'core-js/modules/es.string.match.js';
import 'core-js/modules/es.string.replace.js';
import 'core-js/modules/es.string.trim.js';
import 'core-js/modules/es.string.trim-start.js';

function _wrapRegExp() {
  _wrapRegExp = function (e, r) {
    return new BabelRegExp(e, void 0, r);
  };
  var e = RegExp.prototype,
    r = new WeakMap();
  function BabelRegExp(e, t, p) {
    var o = RegExp(e, t);
    return r.set(o, p || r.get(e)), _setPrototypeOf(o, BabelRegExp.prototype);
  }
  function buildGroups(e, t) {
    var p = r.get(t);
    return Object.keys(p).reduce(function (r, t) {
      var o = p[t];
      if ("number" == typeof o) r[t] = e[o];else {
        for (var i = 0; void 0 === e[o[i]] && i + 1 < o.length;) i++;
        r[t] = e[o[i]];
      }
      return r;
    }, Object.create(null));
  }
  return _inherits(BabelRegExp, RegExp), BabelRegExp.prototype.exec = function (r) {
    var t = e.exec.call(this, r);
    if (t) {
      t.groups = buildGroups(t, this);
      var p = t.indices;
      p && (p.groups = buildGroups(p, this));
    }
    return t;
  }, BabelRegExp.prototype[Symbol.replace] = function (t, p) {
    if ("string" == typeof p) {
      var o = r.get(this);
      return e[Symbol.replace].call(this, t, p.replace(/\$<([^>]+)>/g, function (e, r) {
        var t = o[r];
        return "$" + (Array.isArray(t) ? t.join("$") : t);
      }));
    }
    if ("function" == typeof p) {
      var i = this;
      return e[Symbol.replace].call(this, t, function () {
        var e = arguments;
        return "object" != typeof e[e.length - 1] && (e = [].slice.call(e)).push(buildGroups(e, i)), p.apply(this, e);
      });
    }
    return e[Symbol.replace].call(this, t, p);
  }, _wrapRegExp.apply(this, arguments);
}
function _inherits(subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function");
  }
  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      writable: true,
      configurable: true
    }
  });
  Object.defineProperty(subClass, "prototype", {
    writable: false
  });
  if (superClass) _setPrototypeOf(subClass, superClass);
}
function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };
  return _setPrototypeOf(o, p);
}

const DEFAULT_VALIDATOR_INTERVAL = 500;
const DEFAULT_KEY_ENTRY_INTERVAL = 200;
const POTENTIAL_PAIR = '"_POTENTIAL_EDITOR_KEY_": "_POTENTIAL_EDITOR_VALUE_"';
const POTENTIAL_PAIR_PHRASE = '"_POTENTIAL_PHRASE_KEY_": "_POTENTIAL_PHRASE_VALUE_",';
const POTENTIAL_ITEM = '"_POTENTIAL_EDITOR_ITEM_"';
const POTENTIAL_ITEM_PHRASE = '"_POTENTIAL_PHRASE_ITEM_",';
const debounce = (fn, time) => {
  let timeoutId;
  return wrapper;
  function wrapper(...args) {
    let deferredResolve;
    const promise = new Promise(resolve => {
      deferredResolve = resolve;
    });
    if (timeoutId) {
      clearTimeout(timeoutId);
      timeoutId = null;
    }
    timeoutId = setTimeout(() => {
      timeoutId = null;
      deferredResolve(fn(...args));
    }, time);
    return promise;
  }
};
const debounceKeyEntry = (onKeyEntry, time) => {
  return debounce(onKeyEntry, time);
};
const keyChecker = split => checkKeyEntry(split);
const debouncedKeyEntryFactory = () => {
  let handler;
  const creator = debounceInterval => {
    if (!handler) {
      handler = debounceKeyEntry(keyChecker, debounceInterval);
    }
    return handler;
  };
  return (debounceInterval = DEFAULT_KEY_ENTRY_INTERVAL) => creator(debounceInterval);
};
const debounceValidator = (validator, time) => {
  return debounce(validator, time);
};
const jsonIsValid = json => {
  try {
    return !!JSON.parse(json);
  } catch (e) {
    return false;
  }
};
const debouncedJsonValidatorFactory = () => {
  let validator;
  const creator = debounceInterval => {
    if (!validator) {
      validator = debounceValidator(jsonIsValid, debounceInterval);
    }
    return validator;
  };
  return (debounceInterval = DEFAULT_VALIDATOR_INTERVAL) => creator(debounceInterval);
};
const tryFormatJson = (json, indent, stripTrailingCommas) => {
  const value = stripTrailingCommas ? json.replace(/,\s*([\]}])/g, '$1') : json;
  try {
    return JSON.stringify(JSON.parse(value), null, indent);
  } catch (e) {
    return null;
  }
};
let cachedFont = '400 14px monospace';
const setCanvasFont = inputEl => {
  let computedStyles = null;
  let fontFamilyValue;
  let fontSizeValue;
  let fontSizeUnit;
  let fontWeightValue;
  try {
    computedStyles = inputEl.computedStyleMap();
    const fontFamily = computedStyles.get('font-family');
    fontFamilyValue = (fontFamily === null || fontFamily === void 0 ? void 0 : fontFamily.toString()) || 'monospace';
    // @ts-expect-error this is a CSSUnitValue not a CSSStyleValue
    const fontSize = computedStyles.get('font-size');
    fontSizeValue = fontSize.value;
    fontSizeUnit = fontSize.unit;
    // @ts-expect-error this is a CSSUnitValue not a CSSStyleValue
    const fontWeight = computedStyles.get('font-weight');
    fontWeightValue = fontWeight.value;
  } catch (e) {
    fontFamilyValue = 'monospace';
    fontSizeValue = 14;
    fontSizeUnit = 'px';
    fontWeightValue = 400;
  }
  cachedFont = `${fontWeightValue} ${fontSizeValue}${fontSizeUnit} ${fontFamilyValue}`;
  return computedStyles;
};
const calculateTextWidth = text => {
  // re-use canvas object for better performance
  const canvas = document.querySelector('.evj-calc-canvas');
  if (canvas) {
    const context = canvas.getContext("2d");
    if (context) {
      context.font = cachedFont;
      const metrics = context.measureText(text);
      return metrics.width;
    }
  }
  return text.length * 16;
};
const calculatePosition = (split, inputEl, excludeScroll, yOffset = 0) => {
  const frontLines = split.front.split('\n');
  const lineFront = frontLines[frontLines.length - 1];
  const selectionRow = frontLines.length > yOffset - 1 ? frontLines.length - yOffset : 0;
  const computedStyles = setCanvasFont(inputEl);
  let lineHeightValue = 19.6;
  try {
    if (computedStyles) {
      // @ts-expect-error this is a CSSUnitValue not a CSSStyleValue
      const lineHeight = computedStyles.get('line-height');
      lineHeightValue = lineHeight.value;
    }
  } catch (e) {
    lineHeightValue = 19.6;
  }
  const textSize = calculateTextWidth(lineFront);
  const fullHeight = lineHeightValue * selectionRow;
  return {
    x: textSize,
    y: excludeScroll ? fullHeight - inputEl.scrollTop : fullHeight
  };
};
const updateScrollTop = (inputEl, value) => {
  const split = splitInputAtCursor(inputEl, value);
  const pos = calculatePosition(split, inputEl, false, 3);
  inputEl.scrollTop = pos.y;
};
const setCursorPosition = (inputEl, value, positionChoices) => {
  const position = positionChoices.reduce((currentChoice, item) => {
    if (currentChoice === -1) {
      return item;
    }
    return currentChoice;
  }, -1);
  if (position === -1) {
    return;
  }
  setTimeout(() => {
    inputEl.focus();
    inputEl.selectionStart = position;
    inputEl.selectionEnd = position;
    inputEl.setSelectionRange(position, position);
    updateScrollTop(inputEl, value);
  }, 0);
};
const splitInputAtCursor = (inputEl, value) => {
  const front = value.substring(0, inputEl.selectionStart);
  const back = value.substring(inputEl.selectionStart);
  return {
    front,
    back
  };
};
const tryJsonInsertAtCursor = (splitInput, content, offset = 0) => {
  const value = `${splitInput.front}${content}${splitInput.back}`;
  const result = tryFormatJson(value, 2, false) || tryFormatJson(value, 2, true);
  return result === null ? null : {
    value: result,
    offset
  };
};
const tryJsonReplacementAtCursor = (splitInput, reTest, replacement, offset = 0) => {
  const lines = splitInput.front.split('\n');
  if (!lines.length) {
    return null;
  }
  const lineFront = lines[lines.length - 1];
  const lineBack = splitInput.back.split('\n')[0];
  const lastLine = `${lineFront}${lineBack}`;
  if (!reTest.test(lastLine)) {
    return null;
  }
  return tryJsonInsertAtCursor({
    front: splitInput.front.substring(0, splitInput.front.length - lineFront.length),
    back: splitInput.back.slice(0 - splitInput.back.length + lineBack.length)
  }, lastLine.replace(reTest, replacement), offset);
};
// ^(?!.*(trunk|tags|branches)).*$
const testNakedKey = /^\s*([{]?)([a-z0-9_$-]+)(.*)$/;
const testNakedValue = /^([^:]*:)\s*(?!\s*((([0-9]+[.])?[0-9]+)|true|false|null|(.*".*)))(.*)\s*$/;
const testNakedPair = /^\s*([a-z0-9_$-]+)\s*:\s*([a-z0-9_$-]+[,]?)$/;
const testNakedItem = /^\s*([$a-z][a-z0-9_$-]*)([,]?)$/;
const testNonstringItem = /^\s*((([0-9]+[.])?[0-9]+)|true|false|null)([,]?)$/;
const testStringItem = /^\s*"([^"]*)"([,]?)$/;
const testObject = /^\s*(["]?)([a-z0-9_$-]+)(["]?)\s*(:\s*\{)\s*(})\s*$/;
const testArray = /^\s*(["]?)([a-z0-9_$-]+)(["]?)\s*(:\s*\[)\s*(])\s*$/;
const calculateBestPotentialValue = split => {
  return tryJsonInsertAtCursor(split, `${POTENTIAL_PAIR}}`) || tryJsonInsertAtCursor(split, `${POTENTIAL_ITEM}]`) || tryJsonInsertAtCursor(split, `${POTENTIAL_ITEM}`) || tryJsonInsertAtCursor(split, `${POTENTIAL_PAIR}`) || tryJsonInsertAtCursor(split, '}') || tryJsonInsertAtCursor(split, ']') || tryJsonInsertAtCursor(split, `,${POTENTIAL_PAIR}`) || tryJsonInsertAtCursor(split, `,${POTENTIAL_ITEM}`) || tryJsonInsertAtCursor(split, `${POTENTIAL_PAIR_PHRASE}`) || tryJsonInsertAtCursor(split, `${POTENTIAL_ITEM_PHRASE}`) || tryJsonInsertAtCursor(split, `,${POTENTIAL_PAIR_PHRASE}`) || tryJsonInsertAtCursor(split, `,${POTENTIAL_ITEM_PHRASE}`) || tryJsonReplacementAtCursor(split, testArray, `"$2"$4${POTENTIAL_ITEM}$5`) || tryJsonReplacementAtCursor(split, testArray, `"$2"$4${POTENTIAL_ITEM}$5,`) || tryJsonReplacementAtCursor(split, testObject, `"$2"$4${POTENTIAL_PAIR}$5`) || tryJsonReplacementAtCursor(split, testObject, `"$2"$4${POTENTIAL_PAIR}$5,`) || tryJsonReplacementAtCursor(split, testNonstringItem, `$1,${POTENTIAL_ITEM},`) || tryJsonReplacementAtCursor(split, testStringItem, `"$1",${POTENTIAL_ITEM},`) || tryJsonReplacementAtCursor(split, testNakedPair, `"$1": $2,${POTENTIAL_PAIR}`) || tryJsonReplacementAtCursor(split, testNakedPair, `"$1": $2${POTENTIAL_PAIR}`) || tryJsonReplacementAtCursor(split, testNakedPair, `"$1": "$2",${POTENTIAL_PAIR}`) || tryJsonReplacementAtCursor(split, testNakedPair, `"$1": "$2"${POTENTIAL_PAIR}`) || tryJsonReplacementAtCursor(split, testNakedPair, `"$1": "$2",${POTENTIAL_PAIR_PHRASE}`) || tryJsonReplacementAtCursor(split, testNakedPair, `"$1": "$2"${POTENTIAL_PAIR_PHRASE}`) || tryJsonReplacementAtCursor(split, testNakedKey, `$1"$2"$3,${POTENTIAL_PAIR}`) || tryJsonReplacementAtCursor(split, testNakedKey, `$1"$2"$3${POTENTIAL_PAIR}`) || tryJsonReplacementAtCursor(split, testNakedKey, `$1"$2"$3${POTENTIAL_PAIR}}`) || tryJsonReplacementAtCursor(split, testNakedKey, `$1"$2"$3{${POTENTIAL_PAIR}}`) || tryJsonReplacementAtCursor(split, testNakedKey, `$1"$2"$3: {${POTENTIAL_PAIR}}`) || tryJsonReplacementAtCursor(split, testNakedKey, `$1"$2"$3: {${POTENTIAL_PAIR}},`) || tryJsonReplacementAtCursor(split, testNakedKey, `$1"$2"${POTENTIAL_ITEM}]`) || tryJsonReplacementAtCursor(split, testNakedItem, `"$1"$2,${POTENTIAL_ITEM}`) || tryJsonReplacementAtCursor(split, testNakedItem, `"$1"$2${POTENTIAL_ITEM}`) || tryJsonReplacementAtCursor(split, testNakedItem, '"$1"$2', 2) || tryJsonReplacementAtCursor(split, testNakedKey, '$1"$2"$3', 2) || tryJsonReplacementAtCursor(split, testNakedValue, `$1"$6",${POTENTIAL_PAIR}`) || tryJsonInsertAtCursor(split, '');
};
const scrubPotentialValues = value => value.replace(POTENTIAL_ITEM, '').replace(POTENTIAL_PAIR, '').replace(POTENTIAL_ITEM_PHRASE, '').replace(POTENTIAL_PAIR_PHRASE, '');
const getGroup = (matches, name) => matches.groups ? matches.groups[name] : undefined;
const analyzeJsonLine = line => {
  const matches = line.match( /*#__PURE__*/_wrapRegExp(/^[\s]*(([{[])?(("([^"]+)")(: ))?([{[])?(("([^"]+)")?([a-z0-9.-]+)?)?([}\]])?(,)?)$/, {
    gopen: 2,
    gkey: 4,
    gpair: 6,
    gvalopen: 7,
    gval: 8,
    gclose: 12,
    gdelim: 13
  }));
  if (!matches) {
    const _indentSize = calculateTextWidth(line.substring(0, line.length - line.trimStart().length));
    return {
      indentSize: _indentSize,
      invalid: line.trim()
    };
  }
  const indentSize = calculateTextWidth(matches[0].substring(0, matches[0].length - matches[1].length));
  return {
    indentSize,
    openSymbol: getGroup(matches, 'gopen'),
    key: getGroup(matches, 'gkey'),
    pairDelim: getGroup(matches, 'gpair'),
    valueOpenSymbol: getGroup(matches, 'gvalopen'),
    value: getGroup(matches, 'gval'),
    closeSymbol: getGroup(matches, 'gclose'),
    delim: getGroup(matches, 'gdelim')
  };
};
const closureTest = (char, opener, closer) => char === opener && 1 || char === closer && -1 || 0;
const inTest = (char, negOpener, [opener, closer], enclosureBalance, inEnclosure) => {
  if (char === negOpener || char === closer && enclosureBalance === 1) {
    return false;
  }
  return char === opener && true || inEnclosure;
};
const reduceBalance = (value, initInQuote, initInBrace, initInBracket) => value.split('').reduce(({
  brace,
  bracket,
  quote,
  inQuote,
  inBrace,
  inBracket
}, char) => {
  if (inQuote) {
    return {
      brace,
      bracket,
      quote: quote + (char === '"' ? -1 : 0),
      inQuote: char === '"' ? !inQuote : inQuote,
      inBrace,
      inBracket
    };
  }
  return {
    brace: brace + closureTest(char, '{', '}'),
    bracket: bracket + closureTest(char, '[', ']'),
    quote: quote + closureTest(char, '"', '"'),
    inQuote: char === '"' ? !inQuote : inQuote,
    inBrace: inTest(char, '[', ['{', '}'], brace, inBrace),
    inBracket: inTest(char, '{', ['[', ']'], bracket, inBracket)
  };
}, {
  brace: 0,
  bracket: 0,
  quote: 0,
  inQuote: initInQuote,
  inBrace: initInBrace,
  inBracket: initInBracket
});
const inObjectKey = (frontBalance, backBalance, lineFront) => {
  const openIndex = lineFront.lastIndexOf('{');
  const colonIndex = lineFront.substring(openIndex === -1 ? 0 : openIndex).indexOf(':');
  const inValue = colonIndex !== -1;
  if (frontBalance.inQuote || frontBalance.quote % 2 && backBalance.quote % 2) {
    return frontBalance.brace > 0 && !inValue;
  } else if (frontBalance.inBrace) {
    return !inValue;
  } else if (frontBalance.bracket > 0 && backBalance.bracket < 0) {
    return false;
  } else {
    return frontBalance.brace > 0 && !inValue; // && backBalance.brace < 0;
  }
};
let editorElCache = document.querySelector('.evj-edit');
const testKeyFront = /^([\s]*\s*"?)([a-z0-9_$-]+)$/;
const testKeyBack = /^([a-z0-9_$-]+)(\s*"?:?(.*))$/;
const checkKeyEntry = splitInput => {
  const frontBalance = reduceBalance(splitInput.front, false, false, false);
  const backBalance = reduceBalance(splitInput.back, frontBalance.inQuote, frontBalance.inBrace || !!(frontBalance.quote % 2), frontBalance.inBracket || !!(frontBalance.quote % 2));
  const lines = splitInput.front.split('\n');
  const lineFront = lines[lines.length - 1];
  if (inObjectKey(frontBalance, backBalance, lineFront)) {
    const keyFront = lineFront.replace(testKeyFront, '$2');
    const lineBack = splitInput.back.split('\n')[0];
    const keyBack = lineBack.replace(testKeyBack, '$1');
    if (testKeyFront.test(keyFront)) {
      const inputEl = editorElCache || document.querySelector('.evj-edit');
      editorElCache = inputEl;
      const position = inputEl ? calculatePosition(splitInput, inputEl, true) : {
        x: 0,
        y: 0
      };
      if (testKeyBack.test(keyBack)) {
        return {
          key: `${keyFront}${keyBack}`,
          position
        };
      } else {
        return {
          key: keyFront,
          position
        };
      }
    }
  }
  return null;
};
const setKey = (splitInput, key) => {
  const frontLines = splitInput.front.split('\n');
  const preLines = frontLines.slice(0, frontLines.length - 1);
  const lineFront = frontLines[frontLines.length - 1];
  const preKey = lineFront.replace(testKeyFront, '$1');
  const backLines = splitInput.back.split('\n');
  const postLines = backLines.slice(1);
  const lineBack = backLines[0];
  const postKey = lineBack.replace(testKeyBack, '$2');
  return [...preLines, `${preKey}${key}${postKey}`, ...postLines].join('\n');
};
const enclosures = [['{', '}', (frontBrace, backBrace) => frontBrace + backBrace], ['[', ']', (_f, _b, frontBracket, backBracket) => frontBracket + backBracket], ['"', '"', (_f1, _b1, _f2, _b2, frontQuote, backQuote) => frontQuote - backQuote]];
const complementEnclosure = splitInput => {
  const lastChar = splitInput.front.slice(-1);
  const enclosureCloser = enclosures.find(([_o, c]) => c === lastChar);
  if (enclosureCloser && splitInput.back.slice(0, 1) === lastChar) {
    return complementCloser(enclosureCloser, splitInput);
  }
  const enclosureOpener = enclosures.find(([o]) => o === lastChar);
  if (enclosureOpener) {
    return complementOpener(enclosureOpener, splitInput);
  }
  return null;
};
const complementOpener = (enclosure, splitInput) => {
  const [_openChar, closeChar, isUnbalanced] = enclosure;
  const frontBalance = reduceBalance(splitInput.front, false, false, false);
  const backBalance = reduceBalance(splitInput.back, frontBalance.inQuote, frontBalance.inBrace || !!(frontBalance.quote % 2), frontBalance.inBracket || !!(frontBalance.quote % 2));
  const balance = isUnbalanced(frontBalance.brace, backBalance.brace, frontBalance.bracket, backBalance.bracket, frontBalance.quote, backBalance.quote);
  if (balance > 0) {
    return `${splitInput.front}${closeChar}${splitInput.back}`;
  }
  return null;
};
const complementCloser = (enclosure, splitInput) => {
  const [_openChar, _closeChar, isUnbalanced] = enclosure;
  const frontBalance = reduceBalance(splitInput.front, false, false, false);
  const backBalance = reduceBalance(splitInput.back, frontBalance.inQuote, frontBalance.inBrace || !!(frontBalance.quote % 2), frontBalance.inBracket || !!(frontBalance.quote % 2));
  const balance = isUnbalanced(frontBalance.brace, backBalance.brace, frontBalance.bracket, backBalance.bracket, frontBalance.quote, backBalance.quote);
  if (balance < 0) {
    return `${splitInput.front.slice(0, -1)}${splitInput.back}`;
  }
  return null;
};

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css_248z = ".edit-vs-json {\n  display: block;\n  box-sizing: border-box;\n}\n\n.edit-vs-json .container {\n  position: relative;\n  box-sizing: border-box;\n}\n\n.edit-vs-json textarea::-webkit-scrollbar {\n    background-color: rgb(0,0,0,0);\n    color: rgb(0,0,0,0);\n}\n\n.edit-vs-json textarea::-webkit-scrollbar-button {\n    background-color: rgb(0,0,0,0);\n    color: rgb(0,0,0,0);\n}\n\n.edit-vs-json textarea::-webkit-scrollbar-track {\n    background-color: rgb(0,0,0,0);\n    color: rgb(0,0,0,0);\n}\n\n.edit-vs-json textarea::-webkit-scrollbar-track-piece {\n    background-color: rgb(0,0,0,0);\n    color: rgb(0,0,0,0);\n}\n\n.edit-vs-json textarea::-webkit-scrollbar-thumb {\n    background-color: rgb(0,0,0,0);\n    color: rgb(0,0,0,0);\n}\n\n.edit-vs-json textarea::-webkit-scrollbar-corner {\n    background-color: rgb(0,0,0,0);\n    color: rgb(0,0,0,0);\n}\n\n.edit-vs-json textarea {\n    scrollbar-3dlight-color: rgb(0,0,0,0);\n    -ms-scrollbar-3dlight-color: rgb(0,0,0,0);\n    scrollbar-arrow-color: rgb(0,0,0,0);\n    -ms-scrollbar-arrow-color: rgb(0,0,0,0);\n    scrollbar-base-color: rgb(0,0,0,0);\n    -ms-scrollbar-base-color: rgb(0,0,0,0);\n    scrollbar-color: rgb(0,0,0,0) rgb(0,0,0,0);\n    scrollbar-darkshadow-color: rgb(0,0,0,0);\n    -ms-scrollbar-darkshadow-color: rgb(0,0,0,0);\n    scrollbar-face-color: rgb(0,0,0,0);\n    -ms-scrollbar-face-color: rgb(0,0,0,0);\n    scrollbar-highlight-color: rgb(0,0,0,0);\n    -ms-scrollbar-highlight-color: rgb(0,0,0,0);\n    scrollbar-shadow-color: rgb(0,0,0,0);\n    -ms-scrollbar-shadow-color: rgb(0,0,0,0);\n}\n\n.edit-vs-json .evj-editor {\n    font-family: monospace;\n    font-size: 14px;\n    line-height: 19.6px;\n    font-weight: 400;\n}\n\n.edit-vs-json textarea.evj-edit {\n    color: rgb(0, 0, 0, 0);\n    caret-color: rgb(33, 255, 33, 1);\n    background-color: rgb(0, 0, 0, 0.1);\n    position: absolute;\n    top: 0;\n    left: 0;\n    z-index: 7;\n    font-weight: 800;\n    box-sizing: content-box;\n}\n\n.edit-vs-json textarea.evj-edit:focus {\n    outline: 2px solid deepskyblue;\n    outline-offset: 0;\n}\n\n.edit-vs-json pre.evj-view {\n    background-color: #303030;\n    color: #d0d0d0;\n    z-index: 6;\n    position: relative;\n    overflow: auto;\n    top: 2px;\n    left: 1px;\n}\n\n.edit-vs-json span.line {\n    display: block;\n    height: 19.6px;\n}\n\n.edit-vs-json span.openSymbol {\n    font-weight: 600;\n    color: #efa8ef;\n}\n\n.edit-vs-json span.pairKey {\n    color: #d0d000;\n}\n\n.edit-vs-json span.pairDelim {\n    font-weight: 700;\n}\n\n.edit-vs-json span.valueOpenSymbol {\n    font-weight: 600;\n    color: #efa8ef;\n}\n\n.edit-vs-json span.pairValue {\n    color: orange;\n}\n\n.edit-vs-json span.closeSymbol {\n    font-weight: 600;\n    color: #efa8ef;\n}\n\n.edit-vs-json span.delim {\n    font-weight: 700;\n}\n\n.edit-vs-json span.misc {\n    color: #ff6262;\n}\n\n@supports (-moz-appearance:none) {\n  .edit-vs-json pre.evj-view {\n    top: 6px;\n    left: 2px;\n  }\n}\n\n@supports (-webkit-hyphens: none) {\n  .edit-vs-json .evj-editor {\n        line-height: 19.8px;\n  }\n\n  .edit-vs-json span.line {\n        height: 19.04px;\n  }\n\n  .edit-vs-json textarea.evj-edit {\n        top: 0;\n  }\n\n  .edit-vs-json pre.evj-view {\n        margin-left: -1px;\n        margin-top: 13px;\n  }\n}\n\n";
styleInject(css_248z);

const createValidator = debouncedJsonValidatorFactory();
const createKeyEntryHandler = debouncedKeyEntryFactory();
/**
 * Editor is the edit-vs-json React component that renders a JSON editor
 * @param {string} [width="100%"] the width of the JSON editor control
 * @param {string} [height="100%"] the height of the JSON editor control
 * @param {string} [initValue=""] the initial JSON string value to populate the editor
 * @param {function(value:string, isValid:boolean)} [onChange] the JSON editor change event handler
 * @param {number} [changeDebounceInterval] the debounce interval in ms for the onChange event
 * @param {function(key:string, callback:function)} [onKeyEntry] the object key entry event handler
 * @param {number} [keyEntryDebounceInterval] the debounce interval in ms for the onKeyEntry event
 * @param {boolean} [locked=false] locks the editor when set to true
 * @constructor
 */
const Editor = ({
  width,
  height,
  initValue: _initValue = '',
  onChange,
  changeDebounceInterval,
  onKeyEntry,
  keyEntryDebounceInterval,
  locked
}) => {
  const [value, setValue] = useState(_initValue);
  const editorEl = useRef();
  const viewerEl = useRef();
  const changeBlocked = useRef(false);
  const debouncedValidator = createValidator(changeDebounceInterval);
  const debouncedKeyEntry = createKeyEntryHandler(keyEntryDebounceInterval);
  useEffect(() => {
    const scrollSync = () => {
      if (editorEl.current && viewerEl.current) {
        viewerEl.current.scrollTop = editorEl.current.scrollTop;
      }
    };
    if (editorEl.current) {
      setCanvasFont(editorEl.current);
      editorEl.current.addEventListener('scroll', scrollSync);
    }
    return () => {
      if (editorEl.current) {
        editorEl.current.removeEventListener('scroll', scrollSync);
      }
    };
  }, []);
  const handleChange = event => {
    if (changeBlocked.current) {
      return;
    }
    const inputEl = event.target;
    const split = splitInputAtCursor(inputEl, event.target.value);
    const newValue = complementEnclosure(split);
    if (newValue) {
      setCursorPosition(inputEl, newValue, [split.front.length, inputEl.selectionStart]);
    }
    setValue(newValue || event.target.value);
    if (onKeyEntry) {
      debouncedKeyEntry(split).then(keyEntryResponse => {
        if (keyEntryResponse) {
          const {
            key,
            position
          } = keyEntryResponse;
          onKeyEntry(key, position, newKey => {
            if (newKey) {
              const updatedValue = setKey(split, newKey);
              setValue(updatedValue);
              setCursorPosition(inputEl, updatedValue, [split.front.length + (newKey.length - key.length), inputEl.selectionStart]);
            }
          });
        }
      });
    }
    if (onChange) {
      debouncedValidator(event.target.value).then(result => {
        onChange === null || onChange === void 0 ? void 0 : onChange(event.target.value, result);
      });
    }
  };
  const handleKeyDown = event => {
    const inputEl = event.target;
    if (event.key !== 'Enter' || changeBlocked.current) {
      return;
    }
    changeBlocked.current = true;
    setValue(previousValue => {
      const split = splitInputAtCursor(inputEl, previousValue);
      const newResult = calculateBestPotentialValue(split);
      if (!newResult || !newResult.value) {
        const newValue = `${split.front}\n${split.back}`;
        setCursorPosition(inputEl, newValue, [split.front.length + 1, inputEl.selectionStart]);
        changeBlocked.current = false;
        return newValue;
      } else {
        const updatedValue = scrubPotentialValues(newResult.value) || previousValue;
        setCursorPosition(inputEl, updatedValue, [newResult.value.indexOf(POTENTIAL_PAIR_PHRASE), newResult.value.indexOf(POTENTIAL_ITEM_PHRASE), newResult.value.indexOf(POTENTIAL_PAIR) + newResult.offset, newResult.value.indexOf(POTENTIAL_ITEM) + newResult.offset, inputEl.selectionStart + newResult.offset]);
        changeBlocked.current = false;
        return updatedValue;
      }
    });
  };
  const handleLockedKeyDown = event => {
    event.stopPropagation();
    event.preventDefault();
  };
  const handleLockedChange = event => {
    event.stopPropagation();
    event.preventDefault();
  };
  const handleBlur = () => {
    const newValue = tryFormatJson(value, 2, true);
    if (newValue) {
      setValue(newValue);
      onChange === null || onChange === void 0 ? void 0 : onChange(newValue, true);
    }
  };
  const style = {
    width: width || '100%',
    height: height || '100%',
    opacity: locked ? 0.98 : 1
  };
  const lines = value.split('\n');
  return jsx("div", {
    className: "edit-vs-json",
    children: jsxs("div", {
      className: "container",
      children: [jsx("pre", {
        tabIndex: 0,
        autoFocus: true,
        className: "evj-editor evj-view",
        style: style,
        onClick: e => e.preventDefault(),
        ref: el => viewerEl.current = el,
        children: lines.map((line, index) => {
          const details = analyzeJsonLine(line);
          return details ? jsxs("span", {
            className: "line",
            style: {
              marginLeft: `${details.indentSize}px`
            },
            children: [jsx("span", {
              className: "openSymbol",
              children: details.openSymbol
            }), jsx("span", {
              className: "pairKey",
              children: details.key
            }), jsx("span", {
              className: "pairDelim",
              children: details.pairDelim
            }), jsx("span", {
              className: "valueOpenSymbol",
              children: details.valueOpenSymbol
            }), jsx("span", {
              className: "pairValue",
              children: details.value
            }), jsx("span", {
              className: "closeSymbol",
              children: details.closeSymbol
            }), jsx("span", {
              className: "delim",
              children: details.delim
            }), jsx("span", {
              className: "misc",
              children: details.invalid
            })]
          }, index) : null;
        })
      }), jsx("textarea", {
        className: "evj-editor evj-edit",
        onChange: locked ? handleLockedChange : handleChange,
        onKeyDown: locked ? handleLockedKeyDown : handleKeyDown,
        onBlur: handleBlur,
        style: style,
        value: value,
        ref: el => editorEl.current = el,
        autoComplete: "off",
        autoCorrect: "off",
        autoCapitalize: "off",
        spellCheck: "false"
      }), jsx("canvas", {
        className: "evj-calc-canvas",
        style: {
          visibility: 'hidden'
        }
      })]
    })
  });
};

export { Editor };
